
#ifndef BITREE_H

#define BITREE_H

#include <stdlib.h>
#include "ianstd.h"


/*===============================<o>=====================================

  This software was written by Ian L. Kaplan, Chief Fat Bear, Bear Products
  International.  Use of this software, for any purpose, is granted on two
  conditions: 

   1. This copyright notice must be included with the software
      or any software derived from it.

   2. The risk of using this software is accepted by the user.  No
      warranty as to its usefulness or functionality is provided
      or implied.  The author and Bear Products International provides
      no support.

 ===============================<o>=====================================*/


/* 

   binary_tree class definition

   This class supports insertion and deletion of elements in a simple
   binary tree (e.g., the tree is not self balancing).  The class
   provides two public functions:
  
     tree_insert  - insert values into the binary tree
     tree_delete  - delete values from the binrary tree

   The class derived from this class must supply instances of two virtual
   functions:

     get_tree_mem  - allocate memory for the binary tree
     free_tree_mem - deallocate memory for the binary tree

   Several public functions are also provided for debugging (see bitree.cpp)

 */   


class binary_tree {
 protected:

  typedef struct bi_tree_struct {
    unsigned int key;
    unsigned int val;
    bi_tree_struct *leftptr, *rightptr;
  } bi_tree;

 public:
  typedef bi_tree *bi_tree_ptr;

 private:
   bi_tree_ptr root;

 private: // debug functions
  void bi_tree_print(bi_tree_ptr root, unsigned int indent);
  void print_spaces(unsigned int indent);
  void bi_tree_print_sorted(bi_tree_ptr root);
  unsigned int bi_tree_max_branch( bi_tree_ptr root );
  unsigned int bi_tree_cnt( bi_tree_ptr root );

 public: // debug functions
  void print_tree(void) { bi_tree_print( root, 0 ); }
  void print_sorted(void) { bi_tree_print_sorted( root ); }
  unsigned int max_branch_size(void) { return bi_tree_max_branch(root); }
  unsigned int count_tree(void)  { return bi_tree_cnt(root); }  

 private:
  bi_tree_ptr unlink(bi_tree_ptr &root);
  bi_tree_ptr bi_tree_init(unsigned int key,
			   unsigned int val);
  void bi_tree_insert(bi_tree_ptr &root, 
		      unsigned int key, 
		      unsigned int val);
  unsigned int bi_tree_delete(bi_tree_ptr &root, 
			      unsigned int key );

 public:
  binary_tree(void) { root = NULL; }

  void tree_insert( unsigned int key, 
		    unsigned int val ) 
    { bi_tree_insert(root, key, val); }

  unsigned int tree_delete( unsigned int key )
    { return bi_tree_delete(root, key ); }

  virtual bi_tree_ptr get_tree_mem(void);
  virtual void free_tree_mem( bi_tree_ptr node );
}; // class binary_tree

#endif
