
#ifndef FREELIST_H

#define FREELIST_H

/*===============================<o>=====================================

  This software was written by Ian L. Kaplan, Chief Fat Bear, Bear Products
  International.  Use of this software, for any purpose, is granted on two
  conditions: 

   1. This copyright notice must be included with the software
      or any software derived from it.

   2. The risk of using this software is accepted by the user.  No
      warranty as to its usefulness or functionality is provided
      or implied.  The author and Bear Products International provides
      no support.

 ===============================<o>=====================================*/

#include <stdlib.h>

/* Freelist object.

   This object is designed to support deallocation (free) and reallocation
   of memory objects that are all of a constant size.  The size is set
   in the constructor.  This allows the object to be queried about its
   size.  However, no check is made to make sure that the objects
   added to, or removed from, the free list are the proper size.

   The get_from_free_list function picks an item off the free list
   and returns its address.  If there is no item on the free list,
   NULL is returned.

   The add_to_free_list function adds "addr" to the free list.  The
   virtual function get_mem must be implemented by the class that
   inherit this object to provide memory for the free list chain.

   The list_head chain is the free list.  The chain_head is a chain
   that is kept for unused list "links".
*/


class freelist {

 private: // private typedefs and class variables
   typedef 
     struct list_struct {
       void *addr;
       list_struct *next;
     } list;
   list *list_head;
   list *chain_head;
   unsigned int obj_size;

 public:
   freelist(int obj_syze )
     { obj_size = obj_syze;
       list_head = NULL;
       chain_head = NULL; }

   unsigned int get_obj_size(void) { return obj_size; }
   void add_to_free_list( void *addr );
   void *get_from_free_list(void);

   virtual void *get_mem( unsigned int num_bytes );
}; // class freelist

#endif
