
/*===============================<o>=====================================

  This software was written by Ian L. Kaplan, Chief Fat Bear, Bear Products
  International.  Use of this software, for any purpose, is granted on two
  conditions: 

   1. This copyright notice must be included with the software
      or any software derived from it.

   2. The risk of using this software is accepted by the user.  No
      warranty as to its usefulness or functionality is provided
      or implied.  The author and Bear Products International provides
      no support.

 ===============================<o>=====================================*/

#include <stdio.h>
#include <stdlib.h>
#include "mem.h"



//
// mem_alloc
//
// Allocate memory, either from the free list or from page pool
// memory.  Put block size in the hash table, using the block
// address as the key.
//
void *mem_manage::mem_alloc( unsigned int num_bytes )
{
  void *mem = NULL;
  static int cnt;

  cnt++;

  // search_free_list returns a pointer (if there is memory in the free
  // list) and returns "num_bytes" with the actual size of the allocated
  // memory.  Note that the num_bytes that is placed in the hash table
  // is the actual size of the memory block, which may be larger than
  // the original request.  Potentially this can waste memory.
  //
  mem = search_list( num_bytes );
  if (mem == NULL) {
    mem = page_alloc( num_bytes );
  }
  if (mem != NULL) {
    put_in_hash( mem, num_bytes );
  }
  return mem;
} // mem_manage::mem_alloc



//
// mem_free
//
// Place the memory in the free list.  The size of the block is looked
// up in the hash table.
//
void mem_manage::mem_free( void *addr )
{
  unsigned int size;

  size = get_from_hash( addr );
  if (size == 0) {
    printf("mem_manage::free_mem: could not find addr in alloc list\n");
  }
  else {
    add_to_list( addr, size );
  }
}  // mem_manage::free_mem

