/*===============================<o>=====================================

  This software was written by Ian L. Kaplan, Chief Fat Bear, Bear Products
  International.  Use of this software, for any purpose, is granted on two
  conditions: 

   1. This copyright notice must be included with the software
      or any software derived from it.

   2. The risk of using this software is accepted by the user.  No
      warranty as to its usefulness or functionality is provided
      or implied.  The author and Bear Products International provides
      no support.

 ===============================<o>=====================================*/

#include <stdio.h>
#include <stdlib.h>
#include "ordlist.h"

// ordered_list
//
// Note: The destructor for this class does not deallocate the list
//       (although it does deallocate the sentinal).  This class is 
//       written with the assumption that memory will be deallocated 
//       by the class which inherits this class and provides the memory
//       allocation function get_mem.
//
//       New is used to allocate the sentinal, since the constructor
//       for the memory allocator may not have been called yet.
//
ordered_list::ordered_list(void)
{
  list_elem *tmp;

  // Create an empty "sentinal" structure to start sorted linked list.
  // This simplifies the list search and insert algorithm.
  tmp = new list_elem;
  tmp->size = 0;
  tmp->mem = NULL;
  tmp->next = NULL;
  free_list = tmp;
} // ordered_list constructor



//
// ~ordered_list
//
// Make sure that we delete the sentinal, to avoid a memory leak
//
ordered_list::~ordered_list(void)
{
  delete free_list;
}



//
// search_list
//
// Search the ordered list for a block whose size is greater than
// or equal to the num_bytes argument.  Since the block may be
// larger, num_bytes is returned with the actual block size.
// The function returns the address of the block.
//
void *ordered_list::search_list( unsigned int &num_bytes )
{
  list_elem *listptr, *trail;
  void *retmem = NULL;

  listptr = free_list;
  trail = free_list;
  while (listptr->size < num_bytes && listptr->next != NULL) {
    trail = listptr;
    listptr = listptr->next;
  }
  if (listptr->size >= num_bytes) {
    // unlink the memory from the free list and return it
    num_bytes = listptr->size;
    retmem = listptr->mem;
    trail->next = listptr->next;
  }
  return retmem;
}



//
// add_to_list
//
// Add a memory block to the ordered list.
//
void ordered_list::add_to_list(void *addr, unsigned int size )
{
  list_elem *listptr, *new_block, *trail;

  listptr = free_list;
  trail = free_list;
  while (listptr->size < size && listptr->next != NULL) {
    trail = listptr;
    listptr = listptr->next;
  }
  new_block = (list_elem *)get_mem(sizeof( list_elem ));
  new_block->size = size;
  new_block->mem = addr;
  if (listptr->size <= size) {
    new_block->next = listptr->next;
    listptr->next = new_block;
  }
  else {  // listptr->size > size
    trail->next = new_block;
    new_block->next = listptr;
  }
} // ordered_list::add_to_free_list



//
// print_list
//
// A debug function: print the ordered list.
//
void ordered_list::print_list(void)
{
  list_elem *listptr;

  listptr = free_list;
  while (listptr != NULL) {
    printf("%d (%0x)", listptr->size, listptr->mem );
    listptr = listptr->next;
  } // while
  printf("\n");
} // ordered_list::print_free_list



void *ordered_list::get_mem( unsigned int num_bytes)
{
  printf("ordered_list::get_mem: no mem. allocation function provided\n");
  return NULL;
}

